import { Algorithm } from './algorithm.abstract';
import { MatchQuery } from './match.query';
import { PassengerOrientedCarpoolPathCompleter } from './completer/passenger-oriented-carpool-path.completer';
import { PassengerOrientedGeoFilter } from './filter/passenger-oriented-geo.filter';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { PassengerOrientedSelector } from './selector/passenger-oriented.selector';
import {
  RouteCompleter,
  RouteCompleterType,
} from './completer/route.completer';
import { JourneyCompleter } from './completer/journey.completer';
import { JourneyFilter } from './filter/journey.filter';

export class PassengerOrientedAlgorithm extends Algorithm {
  constructor(
    protected readonly query: MatchQuery,
    protected readonly repository: AdRepositoryPort,
  ) {
    super(query, repository);
    this.selector = new PassengerOrientedSelector(query, repository);
    this.processors = [
      new PassengerOrientedCarpoolPathCompleter(query),
      new RouteCompleter(query, RouteCompleterType.BASIC),
      new PassengerOrientedGeoFilter(query),
      new RouteCompleter(query, RouteCompleterType.DETAILED),
      new JourneyCompleter(query),
      new JourneyFilter(query),
    ];
  }
}
