import {
  ArgumentInvalidException,
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface ScheduleItemProps {
  day: number;
  time: string;
  margin: number;
}

export class ScheduleItem extends ValueObject<ScheduleItemProps> {
  get day(): number {
    return this.props.day;
  }

  get time(): string {
    return this.props.time;
  }

  get margin(): number {
    return this.props.margin;
  }

  protected validate(props: ScheduleItemProps): void {
    if (props.day < 0 || props.day > 6)
      throw new ArgumentOutOfRangeException('day must be between 0 and 6');
    if (props.time.split(':').length != 2)
      throw new ArgumentInvalidException('time is invalid');
    if (
      parseInt(props.time.split(':')[0]) < 0 ||
      parseInt(props.time.split(':')[0]) > 23
    )
      throw new ArgumentInvalidException('time is invalid');
    if (
      parseInt(props.time.split(':')[1]) < 0 ||
      parseInt(props.time.split(':')[1]) > 59
    )
      throw new ArgumentInvalidException('time is invalid');
  }
}
