import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { DefaultParamsProviderPort } from '../core/application/ports/default-params-provider.port';
import { DefaultParams } from '../core/application/ports/default-params.type';
import { AlgorithmType } from '../core/application/types/algorithm.types';

const DRIVER = false;
const PASSENGER = true;
const SEATS_PROPOSED = 3;
const SEATS_REQUESTED = 1;
const DEPARTURE_TIME_MARGIN = 900;
const TIMEZONE = 'Europe/Paris';
const ALGORITHM_TYPE = 'PASSENGER_ORIENTED';
const REMOTENESS = 15000;
const USE_PROPORTION = true;
const PROPORTION = 0.3;
const USE_AZIMUTH = true;
const AZIMUTH_MARGIN = 10;
const MAX_DETOUR_DISTANCE_RATIO = 0.3;
const MAX_DETOUR_DURATION_RATIO = 0.3;
const PER_PAGE = 10;

@Injectable()
export class DefaultParamsProvider implements DefaultParamsProviderPort {
  constructor(private readonly _configService: ConfigService) {}
  getParams = (): DefaultParams => ({
    DRIVER:
      this._configService.get('ROLE') !== undefined
        ? this._configService.get('ROLE') == 'driver'
        : DRIVER,
    SEATS_PROPOSED:
      this._configService.get('SEATS_PROPOSED') !== undefined
        ? parseInt(this._configService.get('SEATS_PROPOSED') as string)
        : SEATS_PROPOSED,
    PASSENGER:
      this._configService.get('ROLE') !== undefined
        ? this._configService.get('ROLE') == 'passenger'
        : PASSENGER,
    SEATS_REQUESTED:
      this._configService.get('SEATS_REQUESTED') !== undefined
        ? parseInt(this._configService.get('SEATS_REQUESTED') as string)
        : SEATS_REQUESTED,
    DEPARTURE_TIME_MARGIN:
      this._configService.get('DEPARTURE_TIME_MARGIN') !== undefined
        ? parseInt(this._configService.get('DEPARTURE_TIME_MARGIN') as string)
        : DEPARTURE_TIME_MARGIN,
    STRICT: this._configService.get('STRICT_FREQUENCY') == 'true',
    TIMEZONE: this._configService.get('TIMEZONE') ?? TIMEZONE,
    ALGORITHM_TYPE:
      AlgorithmType[
        this._configService.get('ALGORITHM_TYPE') as AlgorithmType
      ] ?? AlgorithmType[ALGORITHM_TYPE],
    REMOTENESS:
      this._configService.get('REMOTENESS') !== undefined
        ? parseInt(this._configService.get('REMOTENESS') as string)
        : REMOTENESS,
    USE_PROPORTION:
      this._configService.get('USE_PROPORTION') !== undefined
        ? this._configService.get('USE_PROPORTION') == 'true'
        : USE_PROPORTION,
    PROPORTION:
      this._configService.get('PROPORTION') !== undefined
        ? parseFloat(this._configService.get('PROPORTION') as string)
        : PROPORTION,
    USE_AZIMUTH:
      this._configService.get('USE_AZIMUTH') !== undefined
        ? this._configService.get('USE_AZIMUTH') == 'true'
        : USE_AZIMUTH,
    AZIMUTH_MARGIN:
      this._configService.get('AZIMUTH_MARGIN') !== undefined
        ? parseInt(this._configService.get('AZIMUTH_MARGIN') as string)
        : AZIMUTH_MARGIN,
    MAX_DETOUR_DISTANCE_RATIO:
      this._configService.get('MAX_DETOUR_DISTANCE_RATIO') !== undefined
        ? parseFloat(
            this._configService.get('MAX_DETOUR_DISTANCE_RATIO') as string,
          )
        : MAX_DETOUR_DISTANCE_RATIO,
    MAX_DETOUR_DURATION_RATIO:
      this._configService.get('MAX_DETOUR_DURATION_RATIO') !== undefined
        ? parseFloat(
            this._configService.get('MAX_DETOUR_DURATION_RATIO') as string,
          )
        : MAX_DETOUR_DURATION_RATIO,
    PER_PAGE:
      this._configService.get('PER_PAGE') !== undefined
        ? parseInt(this._configService.get('PER_PAGE') as string)
        : PER_PAGE,
  });
}
