import { Controller, Inject, UsePipes } from '@nestjs/common';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { MatchingPaginatedResponseDto } from '../dtos/matching.paginated.response.dto';
import { QueryBus } from '@nestjs/cqrs';
import { MatchRequestDto } from './dtos/match.request.dto';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { AD_ROUTE_PROVIDER } from '@modules/ad/ad.di-tokens';
import { RouteProviderPort } from '@modules/ad/core/application/ports/route-provider.port';
import { MatchMapper } from '@modules/ad/match.mapper';
import { MatchingResult } from '@modules/ad/core/application/queries/match/match.query-handler';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class MatchGrpcController {
  constructor(
    private readonly queryBus: QueryBus,
    @Inject(AD_ROUTE_PROVIDER)
    private readonly routeProvider: RouteProviderPort,
    private readonly matchMapper: MatchMapper,
  ) {}

  @GrpcMethod('MatcherService', 'Match')
  async match(data: MatchRequestDto): Promise<MatchingPaginatedResponseDto> {
    try {
      const matchingResult: MatchingResult = await this.queryBus.execute(
        new MatchQuery(data, this.routeProvider),
      );
      return new MatchingPaginatedResponseDto({
        id: matchingResult.id,
        data: matchingResult.matches.map((match: MatchEntity) =>
          this.matchMapper.toResponse(match),
        ),
        page: matchingResult.page,
        perPage: matchingResult.perPage,
        total: matchingResult.total,
      });
    } catch (e) {
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
