import { getRedisToken } from '@liaoliaots/nestjs-redis';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { MatchingEntity } from '@modules/ad/core/domain/matching.entity';
import { MatchingNotFoundException } from '@modules/ad/core/domain/matching.errors';
import { MatchingRepository } from '@modules/ad/infrastructure/matching.repository';
import { MatchingMapper } from '@modules/ad/matching.mapper';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';

const mockConfigService = {
  get: jest.fn().mockImplementation((value: string) => {
    switch (value) {
      case 'REDIS_MATCHING_KEY':
        return 'MATCHER:MATCHING';
      case 'REDIS_MATCHING_TTL':
        return 900;
      default:
        return 'some_default_value';
    }
  }),
};

const mockEmptyConfigService = {
  get: jest.fn().mockImplementation(() => ({})),
};

const mockRedis = {
  get: jest
    .fn()
    .mockImplementationOnce(() => null)
    .mockImplementation(
      () =>
        '{"_id":"644a7cb3-6436-4db5-850d-b4c7421d4b97","_createdAt":"2023-09-27T15:19:36.487Z","_updatedAt":"2023-09-27T15:19:36.487Z","props":{"matches":[{"adId":"2dfed880-28ad-4a2f-83d9-b8b45677387b","role":"DRIVER","frequency":"RECURRENT","distance":509967,"duration":17404,"initialDistance":495197,"initialDuration":16589,"distanceDetour":14770,"durationDetour":815,"distanceDetourPercentage":2.98,"durationDetourPercentage":4.91,"journeys":[{"props":{"firstDate":"2024-01-07T00:00:00.000Z","lastDate":"2024-06-30T00:00:00.000Z","journeyItems":[{"props":{"lon":0.683083,"lat":47.503445,"duration":17,"distance":0,"actorTimes":[{"props":{"role":"DRIVER","target":"START","firstDatetime":"2024-01-07T20:00:00.000Z","firstMinDatetime":"2024-01-07T19:45:00.000Z","firstMaxDatetime":"2024-01-07T20:15:00.000Z","lastDatetime":"2024-06-30T20:00:00.000Z","lastMinDatetime":"2024-06-30T19:45:00.000Z","lastMaxDatetime":"2024-06-30T20:15:00.000Z"}}]}},{"props":{"lon":0.364394,"lat":46.607501,"duration":4199,"distance":112695,"actorTimes":[{"props":{"role":"PASSENGER","target":"START","firstDatetime":"2024-01-07T21:30:00.000Z","firstMinDatetime":"2024-01-07T21:15:00.000Z","firstMaxDatetime":"2024-01-07T21:45:00.000Z","lastDatetime":"2024-06-30T21:30:00.000Z","lastMinDatetime":"2024-06-30T21:15:00.000Z","lastMaxDatetime":"2024-06-30T21:45:00.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-07T21:09:59.000Z","firstMinDatetime":"2024-01-07T20:54:59.000Z","firstMaxDatetime":"2024-01-07T21:24:59.000Z","lastDatetime":"2024-06-30T21:09:59.000Z","lastMinDatetime":"2024-06-30T20:54:59.000Z","lastMaxDatetime":"2024-06-30T21:24:59.000Z"}}]}},{"props":{"lon":0.559606,"lat":44.175994,"duration":16975,"distance":503502,"actorTimes":[{"props":{"role":"PASSENGER","target":"FINISH","firstDatetime":"2024-01-08T00:42:55.000Z","firstMinDatetime":"2024-01-08T00:27:55.000Z","firstMaxDatetime":"2024-01-08T00:57:55.000Z","lastDatetime":"2024-07-01T00:42:55.000Z","lastMinDatetime":"2024-07-01T00:27:55.000Z","lastMaxDatetime":"2024-07-01T00:57:55.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-08T00:42:55.000Z","firstMinDatetime":"2024-01-08T00:27:55.000Z","firstMaxDatetime":"2024-01-08T00:57:55.000Z","lastDatetime":"2024-07-01T00:42:55.000Z","lastMinDatetime":"2024-07-01T00:27:55.000Z","lastMaxDatetime":"2024-07-01T00:57:55.000Z"}}]}},{"props":{"lon":0.610873,"lat":44.204195,"duration":17395,"distance":509967,"actorTimes":[{"props":{"role":"DRIVER","target":"FINISH","firstDatetime":"2024-01-08T00:49:55.000Z","firstMinDatetime":"2024-01-08T00:34:55.000Z","firstMaxDatetime":"2024-01-08T01:04:55.000Z","lastDatetime":"2024-07-01T00:49:55.000Z","lastMinDatetime":"2024-07-01T00:34:55.000Z","lastMaxDatetime":"2024-07-01T01:04:55.000Z"}}]}}]}}]},{"adId":"57bc4da9-1ac2-4c63-acc7-5ff1fe6bc380","role":"DRIVER","frequency":"RECURRENT","distance":491989,"duration":18170,"initialDistance":477219,"initialDuration":17355,"distanceDetour":14770,"durationDetour":815,"distanceDetourPercentage":3.1,"durationDetourPercentage":4.7,"journeys":[{"props":{"firstDate":"2024-01-07T00:00:00.000Z","lastDate":"2024-06-30T00:00:00.000Z","journeyItems":[{"props":{"lon":0.683083,"lat":47.503445,"duration":17,"distance":0,"actorTimes":[{"props":{"role":"DRIVER","target":"START","firstDatetime":"2024-01-07T20:10:00.000Z","firstMinDatetime":"2024-01-07T19:55:00.000Z","firstMaxDatetime":"2024-01-07T20:25:00.000Z","lastDatetime":"2024-06-30T20:10:00.000Z","lastMinDatetime":"2024-06-30T19:55:00.000Z","lastMaxDatetime":"2024-06-30T20:25:00.000Z"}}]}},{"props":{"lon":0.364394,"lat":46.607501,"duration":4199,"distance":112695,"actorTimes":[{"props":{"role":"PASSENGER","target":"START","firstDatetime":"2024-01-07T21:30:00.000Z","firstMinDatetime":"2024-01-07T21:15:00.000Z","firstMaxDatetime":"2024-01-07T21:45:00.000Z","lastDatetime":"2024-06-30T21:30:00.000Z","lastMinDatetime":"2024-06-30T21:15:00.000Z","lastMaxDatetime":"2024-06-30T21:45:00.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-07T21:19:59.000Z","firstMinDatetime":"2024-01-07T21:04:59.000Z","firstMaxDatetime":"2024-01-07T21:34:59.000Z","lastDatetime":"2024-06-30T21:19:59.000Z","lastMinDatetime":"2024-06-30T21:04:59.000Z","lastMaxDatetime":"2024-06-30T21:34:59.000Z"}}]}},{"props":{"lon":0.192701,"lat":46.029224,"duration":7195,"distance":190046,"actorTimes":[{"props":{"role":"DRIVER","target":"INTERMEDIATE","firstDatetime":"2024-01-07T22:09:55.000Z","firstMinDatetime":"2024-01-07T21:54:55.000Z","firstMaxDatetime":"2024-01-07T22:24:55.000Z","lastDatetime":"2024-06-30T22:09:55.000Z","lastMinDatetime":"2024-06-30T21:54:55.000Z","lastMaxDatetime":"2024-06-30T22:24:55.000Z"}}]}},{"props":{"lon":0.559606,"lat":44.175994,"duration":17741,"distance":485523,"actorTimes":[{"props":{"role":"PASSENGER","target":"FINISH","firstDatetime":"2024-01-08T01:05:41.000Z","firstMinDatetime":"2024-01-08T00:50:41.000Z","firstMaxDatetime":"2024-01-08T01:20:41.000Z","lastDatetime":"2024-07-01T01:05:41.000Z","lastMinDatetime":"2024-07-01T00:50:41.000Z","lastMaxDatetime":"2024-07-01T01:20:41.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-08T01:05:41.000Z","firstMinDatetime":"2024-01-08T00:50:41.000Z","firstMaxDatetime":"2024-01-08T01:20:41.000Z","lastDatetime":"2024-07-01T01:05:41.000Z","lastMinDatetime":"2024-07-01T00:50:41.000Z","lastMaxDatetime":"2024-07-01T01:20:41.000Z"}}]}},{"props":{"lon":0.610873,"lat":44.204195,"duration":18161,"distance":491989,"actorTimes":[{"props":{"role":"DRIVER","target":"FINISH","firstDatetime":"2024-01-08T01:12:41.000Z","firstMinDatetime":"2024-01-08T00:57:41.000Z","firstMaxDatetime":"2024-01-08T01:27:41.000Z","lastDatetime":"2024-07-01T01:12:41.000Z","lastMinDatetime":"2024-07-01T00:57:41.000Z","lastMaxDatetime":"2024-07-01T01:27:41.000Z"}}]}}]}},{"props":{"firstDate":"2024-01-04T00:00:00.000Z","lastDate":"2024-06-27T00:00:00.000Z","journeyItems":[{"props":{"lon":0.683083,"lat":47.503445,"duration":17,"distance":0,"actorTimes":[{"props":{"role":"DRIVER","target":"START","firstDatetime":"2024-01-04T21:00:00.000Z","firstMinDatetime":"2024-01-04T20:45:00.000Z","firstMaxDatetime":"2024-01-04T21:15:00.000Z","lastDatetime":"2024-06-27T21:00:00.000Z","lastMinDatetime":"2024-06-27T20:45:00.000Z","lastMaxDatetime":"2024-06-27T21:15:00.000Z"}}]}},{"props":{"lon":0.364394,"lat":46.607501,"duration":4199,"distance":112695,"actorTimes":[{"props":{"role":"PASSENGER","target":"START","firstDatetime":"2024-01-04T22:20:00.000Z","firstMinDatetime":"2024-01-04T22:05:00.000Z","firstMaxDatetime":"2024-01-04T22:35:00.000Z","lastDatetime":"2024-06-27T22:20:00.000Z","lastMinDatetime":"2024-06-27T22:05:00.000Z","lastMaxDatetime":"2024-06-27T22:35:00.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-04T22:09:59.000Z","firstMinDatetime":"2024-01-04T21:54:59.000Z","firstMaxDatetime":"2024-01-04T22:24:59.000Z","lastDatetime":"2024-06-27T22:09:59.000Z","lastMinDatetime":"2024-06-27T21:54:59.000Z","lastMaxDatetime":"2024-06-27T22:24:59.000Z"}}]}},{"props":{"lon":0.192701,"lat":46.029224,"duration":7195,"distance":190046,"actorTimes":[{"props":{"role":"DRIVER","target":"INTERMEDIATE","firstDatetime":"2024-01-04T22:59:55.000Z","firstMinDatetime":"2024-01-04T22:44:55.000Z","firstMaxDatetime":"2024-01-04T23:14:55.000Z","lastDatetime":"2024-06-27T22:59:55.000Z","lastMinDatetime":"2024-06-27T22:44:55.000Z","lastMaxDatetime":"2024-06-27T23:14:55.000Z"}}]}},{"props":{"lon":0.559606,"lat":44.175994,"duration":17741,"distance":485523,"actorTimes":[{"props":{"role":"PASSENGER","target":"FINISH","firstDatetime":"2024-01-05T01:55:41.000Z","firstMinDatetime":"2024-01-05T01:40:41.000Z","firstMaxDatetime":"2024-01-05T02:10:41.000Z","lastDatetime":"2024-06-28T01:55:41.000Z","lastMinDatetime":"2024-06-28T01:40:41.000Z","lastMaxDatetime":"2024-06-28T02:10:41.000Z"}},{"props":{"role":"DRIVER","target":"NEUTRAL","firstDatetime":"2024-01-05T01:55:41.000Z","firstMinDatetime":"2024-01-05T01:40:41.000Z","firstMaxDatetime":"2024-01-05T02:10:41.000Z","lastDatetime":"2024-06-28T01:55:41.000Z","lastMinDatetime":"2024-06-28T01:40:41.000Z","lastMaxDatetime":"2024-06-28T02:10:41.000Z"}}]}},{"props":{"lon":0.610873,"lat":44.204195,"duration":18161,"distance":491989,"actorTimes":[{"props":{"role":"DRIVER","target":"FINISH","firstDatetime":"2024-01-05T02:02:41.000Z","firstMinDatetime":"2024-01-05T01:47:41.000Z","firstMaxDatetime":"2024-01-05T02:17:41.000Z","lastDatetime":"2024-06-28T02:02:41.000Z","lastMinDatetime":"2024-06-28T01:47:41.000Z","lastMaxDatetime":"2024-06-28T02:17:41.000Z"}}]}}]}}]}],"query":{"driver":false,"passenger":true,"frequency":"RECURRENT","fromDate":"2024-01-02","toDate":"2024-06-30","schedule":[{"day":0,"time":"21:30","margin":900},{"day":4,"time":"22:20","margin":900}],"seatsProposed":3,"seatsRequested":1,"strict":true,"waypoints":[{"position":0,"lon":0.364394,"lat":46.607501,"houseNumber":"298","street":"Aveue de la liberté","locality":"Buxerolles","postalCode":"86180","country":"France"},{"position":1,"lon":0.559606,"lat":44.175994,"houseNumber":"1","street":"place du 8 mai 1945","locality":"Roquefort","postalCode":"47310","country":"France"}],"algorithmType":"PASSENGER_ORIENTED","remoteness":15000,"useProportion":true,"proportion":0.3,"useAzimuth":true,"azimuthMargin":10,"maxDetourDistanceRatio":0.3,"maxDetourDurationRatio":0.3}},"_domainEvents":[]}',
    ),
  set: jest.fn(),
};

const mockMapper = {
  toDomain: jest.fn().mockImplementation(() => ({
    id: '644a7cb3-6436-4db5-850d-b4c7421d4b97',
    createdAt: '2023-09-27T15:19:36.487Z',
    updatedAt: '2023-09-27T15:19:36.487Z',
    props: [],
  })),
  toPersistence: jest.fn(),
};

const matchingEntity: MatchingEntity = new MatchingEntity({
  id: '644a7cb3-6436-4db5-850d-b4c7421d4b97',
  createdAt: new Date(),
  updatedAt: new Date(),
  props: {
    matches: [
      MatchEntity.create({
        adId: 'dd937edf-1264-4868-b073-d1952abe30b1',
        role: Role.DRIVER,
        frequency: Frequency.PUNCTUAL,
        distance: 356041,
        duration: 12647,
        initialDistance: 348745,
        initialDuration: 12105,
        journeys: [
          {
            firstDate: new Date('2023-09-01'),
            lastDate: new Date('2023-09-01'),
            journeyItems: [
              {
                lon: 6.35484,
                lat: 48.26587,
                duration: 0,
                distance: 0,
                actorTimes: [
                  {
                    role: Role.DRIVER,
                    target: Target.START,
                    firstDatetime: new Date('2023-09-01T07:00:00Z'),
                    firstMinDatetime: new Date('2023-09-01T06:45:00Z'),
                    firstMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                    lastDatetime: new Date('2023-09-01T07:00:00Z'),
                    lastMinDatetime: new Date('2023-09-01T06:45:00Z'),
                    lastMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                  },
                ],
              },
            ],
          },
        ],
        // ...
      }),
    ],
    query: {
      driver: false,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: '2023-09-01',
      toDate: '2023-09-01',
      schedule: [
        {
          day: 5,
          time: '06:40',
          margin: 900,
        },
      ],
      seatsProposed: 3,
      seatsRequested: 1,
      strict: true,
      waypoints: [
        {
          lon: 6.389745,
          lat: 48.32644,
        },
        {
          lon: 6.984567,
          lat: 48.021548,
        },
      ],
      algorithmType: 'PASSENGER_ORIENTED',
      remoteness: 15000,
      useProportion: true,
      proportion: 0.3,
      useAzimuth: true,
      azimuthMargin: 10,
      maxDetourDistanceRatio: 0.3,
      maxDetourDurationRatio: 0.3,
    },
  },
});

describe('Matching repository', () => {
  let matchingRepository: MatchingRepository;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        MatchingRepository,
        {
          provide: getRedisToken('default'),
          useValue: mockRedis,
        },
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        {
          provide: MatchingMapper,
          useValue: mockMapper,
        },
      ],
    }).compile();

    matchingRepository = module.get<MatchingRepository>(MatchingRepository);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(matchingRepository).toBeDefined();
  });
  it('should throw an exception if a matching is not found', async () => {
    await expect(
      matchingRepository.get('644a7cb3-6436-4db5-850d-b4c7421d4b98'),
    ).rejects.toBeInstanceOf(MatchingNotFoundException);
  });
  it('should get a matching', async () => {
    const matching: MatchingEntity = await matchingRepository.get(
      '644a7cb3-6436-4db5-850d-b4c7421d4b97',
    );
    expect(matching.id).toBe('644a7cb3-6436-4db5-850d-b4c7421d4b97');
  });
  it('should save a matching', async () => {
    jest.spyOn(mockRedis, 'set');
    await matchingRepository.save(matchingEntity);
    expect(mockRedis.set).toHaveBeenCalledTimes(1);
  });
});

describe('Matching repository without env vars', () => {
  let secondMatchingRepository: MatchingRepository;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        MatchingRepository,
        {
          provide: getRedisToken('default'),
          useValue: mockRedis,
        },
        {
          provide: ConfigService,
          useValue: mockEmptyConfigService,
        },
        {
          provide: MatchingMapper,
          useValue: mockMapper,
        },
      ],
    }).compile();

    secondMatchingRepository =
      module.get<MatchingRepository>(MatchingRepository);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(secondMatchingRepository).toBeDefined();
  });
  it('should get a matching', async () => {
    const matching: MatchingEntity = await secondMatchingRepository.get(
      '644a7cb3-6436-4db5-850d-b4c7421d4b97',
    );
    expect(matching.id).toBe('644a7cb3-6436-4db5-850d-b4c7421d4b97');
  });
  it('should save a matching', async () => {
    jest.spyOn(mockRedis, 'set');
    await secondMatchingRepository.save(matchingEntity);
    expect(mockRedis.set).toHaveBeenCalledTimes(1);
  });
});
